package edu.sjsu.rhinofaces;

import java.util.logging.Logger;

import javax.el.ELContext;
import javax.el.ELContextEvent;
import javax.el.ELContextListener;
import javax.el.ELResolver;
import javax.el.ExpressionFactory;
//import javax.el.ExpressionFactory;
import javax.faces.application.NavigationHandler;
import javax.faces.application.ViewHandler;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;
import javax.faces.el.ValueBinding;
import javax.faces.el.VariableResolver;
import javax.faces.event.ActionListener;
import javax.servlet.jsp.JspContext;
import javax.servlet.jsp.JspFactory;

import com.sun.faces.application.ApplicationImpl;

/**
   The Rhino Application class.  This wires everything else together,
   so when someting needs to be changed out, it should be done here.
   
   This overrides the reference implementation class.
   
   @author Tom Austin
 */
public class RhinoApplication extends ApplicationImpl
{
   private static Logger logger = Logger.global;

   private ELResolver resolver;
   private RhinoELResolver jsResolver;
   private RhinoActionListener actionListener;
   private RhinoNavigationHandler navHandler;
   private RhinoExpressionFactory expressionFactory;
   private RhinoViewHandler viewHandler;
   
   /**
      Constructor.
    */
   public RhinoApplication()
   {
      super();
      this.jsResolver = new RhinoELResolver();
      super.addELResolver(jsResolver);
      this.resolver = new RhinoCompositeResolver(super.getELResolver(), jsResolver);
      this.viewHandler = new RhinoViewHandler(super.getViewHandler());
      
      //JspFactory.getDefaultFactory().getJspApplicationContext(null).addELResolver(jsResolver);
   }
   
   /**
      Returns the RhinoELResolver.
      
      TODO:It might make more sense to add this to the list of resolvers
         rather than replacing them all.
        
      @see com.sun.faces.application.ApplicationImpl#getELResolver()
    */
   @Override
   public ELResolver getELResolver()
   {
      return this.resolver;
   }
   
   /**
      Returns the RhinoActionListener class.
      @see com.sun.faces.application.ApplicationImpl#getActionListener()
    */
   @Override
   public ActionListener getActionListener()
   {
      //return this.actionListener;
      return new RhinoActionListener(super.getActionListener());
   }
   
   @Override
   public ExpressionFactory getExpressionFactory()
   {
      logger.info("***Getting Expression Factory***");
      if (this.expressionFactory == null)
      {
         this.expressionFactory = new RhinoExpressionFactory(super.getExpressionFactory());
      }
      return this.expressionFactory;
   }
   
   @Override
   @SuppressWarnings("deprecation")
   public MethodBinding createMethodBinding(String arg0, Class[] arg1)
   {
      logger.info("***Creating method binding***");
      return super.createMethodBinding(arg0, arg1);
   }
   
   @Override
   @SuppressWarnings("deprecation")
   public ValueBinding createValueBinding(String arg0)
   {
      logger.info("***Creating value binding***");
      return super.createValueBinding(arg0);
   }
   
   @Override
   @SuppressWarnings("deprecation")
   public VariableResolver getVariableResolver()
   {
      logger.info("***Getting Variable Resolver***");
      return super.getVariableResolver();
   }

   /**
      @return
    */
   public JavaScriptEnvironment getJavaScriptEnvironment(ELContext context)
   {
      return jsResolver.getJavaScriptEnvironment(context);
   }
   
   
   @Override
   public NavigationHandler getNavigationHandler()
   {
      return new RhinoNavigationHandler(super.getNavigationHandler());
   }
   
   @Override
   public ViewHandler getViewHandler()
   {
      logger.info("***Getting view handler***");
      //if (this.viewHandler == null)
      //{
      //   this.viewHandler = new RhinoViewHandler(super.getViewHandler());
      //}
      this.viewHandler.setDelegate(super.getViewHandler());
      return this.viewHandler;
   }
}
